local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:GetPlugin("Options")

local AceGUI 	= LibStub("AceGUI-3.0")
local UIF		= LibStub("LibGUIFactory-1.0"):GetFactory("RW")
local L 		= LibStub("AceLocale-3.0"):GetLocale("RW2-Options")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local Utils 	= RW.Utils

function plugin:OptionsBoss()
	local LBB = RW.LBB
	local t = {}
	
	local bossList = {}
	local bossListLastSelected = {}
	local bossHeaders = {}
	local addedOptions = {}	
	
	------------------------------
	-- Option Details
	
	local function showDetailTab(tabs, event, group)
		tabs:ReleaseChildren()
		local baseObj = tabs:GetUserData("baseObj")
		
		local scroll = AceGUI:Create("ScrollFrame")
		scroll:SetLayout("Flow")	
		
		local added = false
		for i, obj in pairs(baseObj.mod.AlertObjects[group]) do
			if obj.base == baseObj then
				added = true
				scroll:AddChildren(obj:GetOption())
			end
		end
		tabs:AddChild(scroll)
	end
	
	-- Returns the tabs for the details about the selected option
	local tabs = {}
	local function getOptionTabs(baseObj)
		wipe(tabs)
		for i, data in pairs(baseObj.mod.AlertObjects) do
			local c = false
			for _, obj in pairs(data) do
				if obj.base == baseObj then
					c = true
					break
				end
			end
			if c then 
				tinsert(tabs, {value = i, text = LB[i]})
			end
		end
		return tabs
	end
	
	local function showHideDetails(baseObj)
		if DetailsFrame then 
			DetailsFrame:Release()
			DetailsFrame = nil
		end
			
		local optText = type(baseObj.id) == "number" and Utils.SpellName[baseObj.id] or baseObj.id
			
		DetailsFrame = UIF:Frame(optText)
		DetailsFrame:SetLayout("Fill")
		DetailsFrame.frame:ClearAllPoints()
		if OptionsFrame then
			DetailsFrame.frame:SetPoint("LEFT", OptionsFrame.frame, "RIGHT", 5, 0)
			DetailsFrame.frame:SetFrameLevel(OptionsFrame.frame:GetFrameLevel() + 1)
		else
			DetailsFrame.frame:SetPoint("CENTER")
			DetailsFrame.frame:SetFrameLevel(50)
		end
		DetailsFrame:SetWidth(500)
		DetailsFrame:SetHeight(450)
		DetailsFrame:SetCallback("OnClose", function(widget)
			widget:ReleaseChildren()
			widget:Release()
			DetailsFrame = nil
		end)
		
		local t = getOptionTabs(baseObj)
		local tab = UIF:TabGroup()
		tab:SetLayout("Fill")
		tab:SetTabs(t)
		tab:SetUserData("baseObj", baseObj)
		tab:SetCallback("OnGroupSelected", showDetailTab)
		tab:SelectTab(t[1] and t[1].value or nil)
		tab:SetCallback("OnTabEnter", function(widget, event, key, tab)
				GameTooltip:SetOwner(widget.frame)
				GameTooltip:AddLine(LB[key])
				GameTooltip:AddLine(LB[key.."_DESC"])
				GameTooltip:Show()
			end)
		tab:SetCallback("OnTabLeave", function() GameTooltip:Hide() end)
		
		DetailsFrame:AddChild(tab)
		DetailsFrame:Show()
	end
	
	------------------------------
	-- Boss Detials
	
	local function getMasterValue(baseObj)
		if baseObj.type == "bool" then
			return baseObj.mod.DB[baseObj.var]
		else
			return baseObj.mod.DB[baseObj.id].enabled
		end
	end
	
	local function toggleMaster(check, event, value)
		local baseObj = check:GetUserData("baseObj")
		
		if baseObj.type == "bool" then
			baseObj.mod.DB[baseObj.var] = not baseObj.mod.DB[baseObj.var]
		else
			baseObj.mod.DB[baseObj.id].enabled = not baseObj.mod.DB[baseObj.id].enabled
		end
		
		if DetailsFrame then
			showHideDetails(baseObj)
		end
	end
	
	local function GetOptionGroup(opt, boss)
		local baseObj = boss:GetBaseAlertObject(opt)
		--[===[@debug@
		if not baseObj then
			--error(("%s: No option connected to %q"):format(boss:GetName(), opt))
		end
		--@end-debug@]===]		
	
		local optText = (baseObj and baseObj.displayText) or (type(opt) == "number" and Utils.SpellName[opt] or opt)
	
		local group = AceGUI:Create("SimpleGroup") -- UIF:InlineGroup2() -- 
		group:SetLayout("Flow")
		group:SetFullWidth(true)
		--group:SetRelativeWidth(0.5)
		--group:SetAutoAdjustHeight(false)
		--group:SetHeight(80)		
		
		if baseObj then
			
			local check = AceGUI:Create("CheckBox")
			check:SetLabel("")
			check:SetUserData("baseObj", baseObj)
			check:SetValue(getMasterValue(baseObj))
			check:SetCallback("OnValueChanged", toggleMaster)
			check:SetRelativeWidth(0.08)
			group:AddChild(check)
		
			local label = UIF:NormalText("", nil, nil, nil, nil, true)
			label:SetImage(baseObj.icon, 0.08, 0.92, 0.08, 0.92)
			label:SetImageSize(25, 25)
			label:SetFullWidth(false)
			label:SetRelativeWidth(0.12)
			label:SetUserData("baseObj", baseObj)
			
			if baseObj.type ~= "bool" then
				label:SetCallback("OnClick", function(self, event, button)
					local baseObj = self:GetUserData("baseObj")
					if not baseObj.spell then return end
					local spell = baseObj.spell
					if button == "LeftButton" then
						if IsShiftKeyDown() then
							local ChatEdit =  ChatEdit_GetActiveWindow()
							if not ChatEdit then ChatEdit = ChatEdit_GetLastActiveWindow() end
							if not ChatEdit then ChatEdit = ChatFrame1EditBox end
							ChatEdit:Show()
							ChatEdit_FocusActiveWindow()
							ChatEdit:Insert("\124cff71d5ff\124Hspell:"..spell.."\124h["..Utils.SpellName[spell].."]\124h\124r")
						end
					end
				end)
			end
			
			if baseObj.spell then
				label:SetCallback("OnEnter", function(self)
					GameTooltip:SetOwner(self.frame)
					GameTooltip:SetHyperlink("spell:"..baseObj.spell)
					GameTooltip:AddLine(" ")
					GameTooltip:AddLine("["..L.SHIFT_CLICK_LINK_DESC.."]")
					GameTooltip:Show()
				end)
				label:SetCallback("OnLeave", function()
					GameTooltip:Hide()
				end)
			end			
			group:AddChild(label)
			
			label = UIF:NormalText(optText)
			label:SetFullWidth(false)
			label:SetRelativeWidth(0.29)
			group:AddChild(label)			
			
			local iconGroup = AceGUI:Create("SimpleGroup")
			iconGroup:SetLayout("Flow")
			iconGroup:SetFullWidth(false)
			iconGroup:SetRelativeWidth(0.3)
			group:AddChild(iconGroup)
			
			local iconObj = boss:GetIconObject(opt)
			if iconObj then
				
				for i,icn in ipairs(iconObj.orderedMarks) do
					local icon = AceGUI:Create("Icon")
					icon:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(Utils.Constants.IconCoords[icn]))
					icon:SetImageSize(18, 18)
					icon:SetHeight(20)
					icon:SetWidth(25)
					iconGroup:AddChild(icon)
				end
			end
			
			local c = false
			for _, objType in pairs(baseObj.mod.AlertObjects) do
				for _, obj in pairs(objType) do
					if obj.base == baseObj then
						c = true
						break
					end
				end
				if c then break end
			end
			if c then 
				local button = UIF:Button("Details", function(widget, event)
				local baseObj = widget:GetUserData("baseObj")
					showHideDetails(baseObj)
				end)
				button:SetRelativeWidth(0.2)
				button:SetUserData("baseObj", baseObj)
				group:AddChild(button)
			end			
		end
		
		return group
	end
	
	local sortCat = {}
	function plugin.SelectBoss(drop, event, group)
		local boss
		if drop.ReleaseChildren then
			drop:ReleaseChildren()
		
			local modName = drop:GetUserData("modName")
			boss = RW:IndexedBossesByPack(modName)[group]
			bossListLastSelected[modName] = group
			drop:SetUserData("boss", boss)
		else
			boss = RW:GetBoss(drop:GetName())
			if not OptionsFrame then
				OptionsFrame = event
			end
		end		
		
		local scroll = AceGUI:Create("SimpleGroup")
		scroll:SetFullWidth(true)
		if drop.AddChild then
			drop:AddChild(scroll)
		end
		
		local sgroup = UIF:InlineGroup(" ")
		scroll:AddChild(sgroup)
		
		--[===[@debug@
		if not LBB[boss:GetName()] then
			print("Could not find in Babble-Boss", boss:GetName())
		end
		--@end-debug@]===]
		local label = UIF:MainTitleLabel(LBB[boss:GetName()] or boss:GetName())
		label:SetImage(boss.BossIcon)
		sgroup:AddChild(label)
		if rawget(boss.L, "info") then
			label = UIF:NormalText(rawget(boss.L, "info"))		
			sgroup:AddChild(label)
		end
		
		local enableCheck = UIF:CheckBox(LB.ENABLED, boss.db.profile, "enabled")
		enableCheck:SetFullWidth(false)
		enableCheck:SetHeight(35)
		sgroup:AddChild(enableCheck)
		
		local check = UIF:CheckBox(L.SORT_OPTIONS, boss.db.profile, "SortBossCategories")
		check:SetFullWidth(false)
		check:SetUserData("TriggerGlobalUpdate", true)
		sgroup:AddChild(check)
		
		if not boss.SupressGlobalHealthFrames then
			check = UIF:CheckBox(L.SHOW_BOSS_HEALTHFRAMES, boss.db.profile, "AutoHealthFrames")
			check:SetFullWidth(false)
			sgroup:AddChild(check)
		end
		
		if boss.Options then
			-- Build options titles
			if boss.Options.Categories then
				for i, cat in pairs(boss.Options.Categories) do
					local group = UIF:InlineGroup(cat.text)
					group:SetLayout("Flow")
					scroll:AddChild(group)
					
					label = UIF:NormalText(LB.ENABLED, nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.1)
					group:AddChild(label)
					
					label = UIF:NormalText(LB.ICON, nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.1)
					group:AddChild(label)
					
					label = UIF:NormalText(LB.NAME, nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.29)
					group:AddChild(label)
					
					label = UIF:NormalText(LB.RAID_ICON, nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.3)
					group:AddChild(label)
					
					label = UIF:NormalText(L.DETAILS, nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.2)
					group:AddChild(label)
					
					
					wipe(sortCat)
					for j, opt in pairs(cat) do
						tinsert(sortCat, opt)
					end
					if boss.db.profile.SortBossCategories then						
						table.sort(sortCat, function(a, b) 
							local sa, sb = a, b
							if type(sa) == "number" then sa = Utils.SpellName[sa] end
							if type(sb) == "number" then sb = Utils.SpellName[sb] end
							return a < b
						end)
					end
					
					for j, opt in ipairs(sortCat) do
						local optGroup = GetOptionGroup(opt, boss)
						if optGroup then
							group:AddChildren(optGroup)
						end
					end
					wipe(sortCat)
				end
			end				
		end		
		
		scroll.parent:DoLayout()
		if scroll.parent.parent then
			scroll.parent.parent:DoLayout()
		end
		return scroll
	end	
	
	local addedSeps = {}
	for modName in RWL:IterateBossMods() do
		local info = RWL:GetModInfo(modName)
		local orderNum = tonumber(info.order or 1)
		local modded = orderNum - orderNum % 100
		if not addedSeps[modded] then	
			addedSeps[modded] = true
			if modded == 100 then
				tinsert(t, function() return {{text = "Mods", value = "Mods", order = 2}, {text = "--Vanilla--", value = "SepCanilla", order = math.abs(1000 - 199), disabled = true}}, function() end end)
			elseif modded == 300 then
				tinsert(t, function() return {{text = "Mods", value = "Mods", order = 2}, {text = "--Wrath of the Lichking--", value = "SepWOTLK", order = math.abs(1000 - 399), disabled = true}}, function() end end)
			elseif modded == 400 then
				tinsert(t, function() return {{text = "Mods", value = "Mods", order = 2}, {text = "--Cataclysm--", value = "SepCata", order = math.abs(1000 - 499), disabled = true}}, function() end end)
			end
		end
		if not IsAddOnLoaded(info.modID) then
			tinsert(t, function()
				return {{text = "Mods", value = "Mods", order = 2}, {text = modName, value = info.modID, order = math.abs(1000 - (tonumber(info.order) or 0))}}, function()
					self = plugin
					
					local group = AceGUI:Create("SimpleGroup")
					group:SetLayout("Flow")
					group:SetFullWidth(true)
					
					local title = UIF:Title(modName)
					title:SetImage(info.icon)
					group:AddChild(title)
					group:AddChild(UIF:Text1("This mod is not loaded yet. Press the Load Mod button to load it"))
					group:AddChild(UIF:NewLine(10))
					group:AddChild(UIF:Button("Load "..modName, function()
						RWL:LoadByName(modName)		
					end))
					
					return group
				end,
				info.icon
			end)
		end
	end
	
	for modName in RW:IterateBossPacks() do
		local info = RWL:GetModInfo(modName)
		tinsert(t, function()
			return {{text = "Mods", value = "Mods", order = 2}, {text = modName, value = info.modID, order = math.abs(1000 - (tonumber(info.order) or 0))}}, function()
				self = plugin				

				local haveCats = false
				local nameList
				for i, boss in RW:IterateBossesByPack(modName) do
					nameList = nameList or {}
					if boss.subGroup then
						haveCats = true
					end
					nameList[i] = {value = i, text = (LBB[boss:GetName()] or boss:GetName()), icon = boss.BossIcon}
				end
				
				if not haveCats then
					local dropdown = UIF:DropdownGroup()
					dropdown:SetDropdownWidth(300)
					dropdown:SetLayout("Flow")
					dropdown:SetUserData("modName", modName)
					dropdown:SetFullHeight(true)
					dropdown:SetFullWidth(true)
					dropdown:SetGroupList(nameList)
					dropdown:SetCallback("OnGroupSelected", plugin.SelectBoss)
					if bossListLastSelected[modName] and nameList[bossListLastSelected[modName]] then
						dropdown:SetGroup(bossListLastSelected[modName])
					else
						dropdown:SetGroup(1)
					end								
					
					return dropdown
				else
					local group = AceGUI:Create("SimpleGroup")
					group:SetFullWidth(true)
					group:SetLayout("Flow")
				
					for i, subCat in pairs(RW:GetSubCategories(modName)) do					
						local label = UIF:MainTitleLabel(subCat.name, nil, nil, nil, nil, true)
						label:SetCallback("OnClick", function(widget)
							plugin.OptionsTree:SetSelected("Mods\001"..info.modID.."\001"..subCat.name)
						end)
						label:SetImage(subCat.icon)
						group:AddChild(label)						
					end
					
					return group
				end
			end,
			info.icon
		end)
		
		for i, subCat in pairs(RW:GetSubCategories(modName)) do
			tinsert(t, function()
				return {
					{text = "Mods", value = "Mods", order = 2}, 
					{text = modName, value = info.modID, order = math.abs(1000 - (tonumber(info.order) or 0))},
					{text = subCat.name, value = subCat.name}
				},
				function() 
					local nameList = {}
					local firstID 
					for i, boss in RW:IterateBossesByPack(modName) do
						if boss.subGroup == subCat.name then
							if not firstID then firstID = i end
							nameList[i] = {value = i, text = (LBB[boss:GetName()] or boss:GetName()), icon = boss.BossIcon}
						end
					end
					local dropdown = UIF:DropdownGroup()
					dropdown:SetDropdownWidth(300)
					dropdown:SetLayout("Flow")
					dropdown:SetUserData("modName", modName)
					dropdown:SetFullHeight(true)
					dropdown:SetFullWidth(true)
					dropdown:SetGroupList(nameList)
					dropdown:SetCallback("OnGroupSelected", plugin.SelectBoss)
					if bossListLastSelected[modName] and nameList[bossListLastSelected[modName]] then
						dropdown:SetGroup(bossListLastSelected[modName])
					else
						dropdown:SetGroup(firstID)
					end								
					
					return dropdown
				end,
				subCat.icon
			end)
		end
	end
	
	return t
end